@@ -20,10 +20,16 @@ class GenericRunnerSettings(error: String => Unit) extends Settings(error) {
       "guess")
 
   val loadfiles =
+    MultiStringSetting(
+      "-I",
+      "file",
+      "load a file line-by-line")
+
+  val pastefiles =
     MultiStringSetting(
       "-i",
       "file",
-      "load a file (assumes the code is given interactively)")
+      "paste a file")
 
   val execute =
     StringSetting(
@@ -39,5 +45,8 @@ class GenericRunnerSettings(error: String => Unit) extends Settings(error) {
 
   val nc = BooleanSetting(
       "-nc",
-      "do not use the fsc compilation daemon") withAbbreviation "-nocompdaemon"
+      "do not use the fsc compilation daemon") withAbbreviation "-nocompdaemon" withPostSetHook((x: BooleanSetting) => {_useCompDaemon = !x.value })
+
+  private[this] var _useCompDaemon = true
+  def useCompDaemon: Boolean = _useCompDaemon
 }
