@@ -89,6 +89,7 @@ trait JavaScanners extends ast.parser.ScannersCommon {
       javanme.ELSEkw         -> ELSE,
       javanme.ENUMkw         -> ENUM,
       javanme.EXTENDSkw      -> EXTENDS,
+      javanme.FALSEkw        -> FALSE,
       javanme.FINALkw        -> FINAL,
       javanme.FINALLYkw      -> FINALLY,
       javanme.FLOATkw        -> FLOAT,
@@ -118,6 +119,7 @@ trait JavaScanners extends ast.parser.ScannersCommon {
       javanme.THROWkw        -> THROW,
       javanme.THROWSkw       -> THROWS,
       javanme.TRANSIENTkw    -> TRANSIENT,
+      javanme.TRUEkw         -> TRUE,
       javanme.TRYkw          -> TRY,
       javanme.VOIDkw         -> VOID,
       javanme.VOLATILEkw     -> VOLATILE,
