@@ -452,10 +452,10 @@ abstract class ICodeReader extends ClassfileParser {
 
         case JVM.ishl        => code.emit(CALL_PRIMITIVE(Shift(LSL, INT)))
         case JVM.lshl        => code.emit(CALL_PRIMITIVE(Shift(LSL, LONG)))
-        case JVM.ishr        => code.emit(CALL_PRIMITIVE(Shift(LSR, INT)))
-        case JVM.lshr        => code.emit(CALL_PRIMITIVE(Shift(LSR, LONG)))
-        case JVM.iushr       => code.emit(CALL_PRIMITIVE(Shift(ASR, INT)))
-        case JVM.lushr       => code.emit(CALL_PRIMITIVE(Shift(ASR, LONG)))
+        case JVM.ishr        => code.emit(CALL_PRIMITIVE(Shift(ASR, INT)))
+        case JVM.lshr        => code.emit(CALL_PRIMITIVE(Shift(ASR, LONG)))
+        case JVM.iushr       => code.emit(CALL_PRIMITIVE(Shift(LSR, INT)))
+        case JVM.lushr       => code.emit(CALL_PRIMITIVE(Shift(LSR, LONG)))
         case JVM.iand        => code.emit(CALL_PRIMITIVE(Logical(AND, INT)))
         case JVM.land        => code.emit(CALL_PRIMITIVE(Logical(AND, LONG)))
         case JVM.ior         => code.emit(CALL_PRIMITIVE(Logical(OR, INT)))
