@@ -5,10 +5,6 @@
 
 package scala.man1
 
-/**
- *  @author Stephane Micheloud
- *  @version 1.0
- */
 object scalac extends Command {
   import _root_.scala.tools.docutil.ManPage._
 
@@ -77,7 +73,7 @@ object scalac extends Command {
           SeqPara(
             "Specify where to find user class files (on Unix-based systems " &
             "a colon-separated list of paths, on Windows-based systems, a " &
-            "semicolon-separate list of paths). This does not override the " &
+            "semicolon-separated list of paths). This does not override the " &
             "built-in (" & Mono("\"boot\"") & ") search path.",
             "The default class path is the current directory. Setting the " &
             Mono("CLASSPATH") & " variable or using the " & Mono("-classpath") & " " &
@@ -152,11 +148,12 @@ object scalac extends Command {
           CmdOption("sourcepath", Argument("path")),
           "Specify location(s) of source files."),
         Definition(
-          CmdOptionBound("target:", "{jvm-1.5,jvm-1.6,jvm-1.7}"),
-	  SeqPara(
+          CmdOptionBound("target:", "{jvm-1.5,jvm-1.6,jvm-1.7,jvm-1.8}"),
+          SeqPara(
             Mono("\"jvm-1.5\"") & " target JVM 1.5 (deprecated),",
             Mono("\"jvm-1.6\"") & " target JVM 1.6 (default),",
-            Mono("\"jvm-1.7\"") & " target JVM 1.7,")),
+            Mono("\"jvm-1.7\"") & " target JVM 1.7,",
+            Mono("\"jvm-1.8\"") & " target JVM 1.8,")),
         Definition(
           CmdOption("toolcp", Argument("path")),
           "Add to the runner classpath."),
@@ -195,7 +192,7 @@ object scalac extends Command {
         Definition(
           CmdOption("Xcheckinit"),
           "Wrap field accessors to throw an exception on uninitialized access."),
-	Definition(
+        Definition(
           CmdOption("Xdev"),
           "Enable warnings for developers working on the Scala compiler"),
         Definition(
@@ -211,7 +208,7 @@ object scalac extends Command {
         Definition(
           CmdOption("Xfatal-warnings"),
           "Fail the compilation if there are any warnings."),
-	Definition(
+        Definition(
           CmdOption("Xfull-lubs"),
           "Retain pre 2.10 behavior of less aggressive truncation of least upper bounds."),
         Definition(
@@ -223,25 +220,25 @@ object scalac extends Command {
         Definition(
           CmdOption("Xlint"),
           "Enable recommended additional warnings."),
-	Definition(
+        Definition(
           CmdOption("Xlog-free-terms"),
           "Print a message when reification creates a free term."),
-	Definition(
+        Definition(
           CmdOption("Xlog-free-types"),
           "Print a message when reification resorts to generating a free type."),
-	Definition(
+        Definition(
           CmdOption("Xlog-implicit-conversions"),
           "Print a message whenever an implicit conversion is inserted."),
         Definition(
           CmdOption("Xlog-implicits"),
           "Show more detail on why some implicits are not applicable."),
-	Definition(
+        Definition(
           CmdOption("Xlog-reflective-calls"),
           "Print a message when a reflective method call is generated."),
-	Definition(
+        Definition(
           CmdOptionBound("Xmacro-settings:", Argument("option")),
           "Custom settings for macros."),
-	Definition(
+        Definition(
           CmdOption("Xmain-class", Argument("path")),
           "Class for manifest's Main-Class entry (only useful with -d <jar>)."),
         Definition(
@@ -253,7 +250,7 @@ object scalac extends Command {
         Definition(
           CmdOption("Xno-forwarders"),
           "Do not generate static forwarders in mirror classes."),
-	Definition(
+        Definition(
           CmdOption("Xno-patmat-analysis"),
           "Don't perform exhaustivity/unreachability analysis. Also, ignore " & MItalic("@switch") & " annotation."),
         Definition(
@@ -311,15 +308,20 @@ object scalac extends Command {
         Definition(
           CmdOptionBound("Xsource:", Argument("version")),
           "Treat compiler input as Scala source for the specified version, see SI-8126."),
-	Definition(
+        Definition(
           CmdOption("Xsource-reader", Argument("classname")),
           "Specify a custom method for reading source files."),
-	Definition(
+        Definition(
           CmdOption("Xstrict-inference"),
           "Don't infer known-unsound types."),
         Definition(
           CmdOption("Xverify"),
           "Verify generic signatures in generated bytecode (asm backend only)."),
+        Definition(
+          CmdOptionBound("Xxml:", "{coalescing}"),
+          SeqPara(
+            "Configure XML parsing.",
+            Mono("\"coalescing\"") & " convert PCData to Text and coalesce sibling nodes (default in 2.11).")),
         Definition(
           CmdOption("Y"),
           "Print a synopsis of private options.")
@@ -334,34 +336,34 @@ object scalac extends Command {
         Definition(
           MItalic("namer"),
           "resolve names, attach symbols to named trees"),
-	Definition(
+        Definition(
           MItalic("packageobjects"),
           "load package objects"),
-	Definition(
+        Definition(
           MItalic("typer"),
           "the meat and potatoes: type the trees"),
         Definition(
           MItalic("patmat"),
           "translate match expressions"),
-	Definition(
+        Definition(
           MItalic("superaccessors"),
           "add super accessors in traits and nested classes"),
-	Definition(
+        Definition(
           MItalic("extmethods"),
           "add extension methods for inline classes"),
-	Definition(
+        Definition(
           MItalic("pickler"),
           "serialize symbol tables"),
         Definition(
           MItalic("refchecks"),
           "reference/override checking, translate nested objects"),
-	Definition(
+        Definition(
           MItalic("selectiveanf"),
           "ANF pre-transform for " & MItalic("@cps") & " (CPS plugin)"),
-	Definition(
+        Definition(
           MItalic("selectivecps"),
           MItalic("@cps") & "-driven transform of selectiveanf assignments (CPS plugin)"),
-	Definition(
+        Definition(
           MItalic("uncurry"),
           "uncurry, translate function values to anonymous classes"),
         Definition(
@@ -449,11 +451,7 @@ object scalac extends Command {
 
           "With Java 1.5 (or newer) one may for example configure the " &
           "memory usage of the JVM as follows: " &
-          Mono("JAVA_OPTS=\"-Xmx512M -Xms16M -Xss16M\""),
-
-          "With " & Link("GNU Java", "http://gcc.gnu.org/java/") & " one " &
-          "may configure the memory usage of the GIJ as follows: " &
-          Mono("JAVA_OPTS=\"--mx512m --ms16m\"")
+          Mono("JAVA_OPTS=\"-Xmx512M -Xms16M -Xss16M\"")
         ))))
 
   val examples = Section("EXAMPLES",
