@@ -26,8 +26,7 @@ private[runtime] class TwoWayCache[J, S] {
   private object SomeRef {
     def unapply[T](optRef: Option[WeakReference[T]]): Option[T] =
       if (optRef.nonEmpty) {
-        val result = optRef.get.get
-        if (result != null) Some(result) else None
+        Option(optRef.get.get)
       } else None
   }
 
