@@ -12,7 +12,7 @@ object Test {
     "A doubly important ${foo * 2} message!"
   }
   def h = s"Try using '$$bar' instead."  // no warn
-  def i = s"Try using '${ "$bar" }' instead."  // no warn on space test
+  def i = s"Try using '${ "$bar" }' instead."  // was: no warn on space test
   def j = s"Try using '${ "something like $bar" }' instead."  // warn
   def k = f"Try using '$bar' instead."  // no warn on other std interps
 }
