@@ -12,5 +12,5 @@ abstract class A {
 class B extends A {
   type MyObj = ObjImpl
   val myString:   S = "hello"
-  val realString: String = myString   // error: type missmatch
+  val realString: String = myString   // error: type mismatch
 }
