@@ -5,16 +5,16 @@ object ExcludedPrefix1 {
   case
   object
   BLAH
-  
+
   val
   a = 1
-  
+
   var
   b = 2
-  
+
   def
   c = 23
-  
+
   private
   def
   d = 23
@@ -22,20 +22,20 @@ object ExcludedPrefix1 {
   lazy
   val
   e = 23
-  
+
   private
   type
   f = Int
-  
+
   val
   g,
   h = 23
-  
+
   val
   (i,
    j) = (0, 0)
-   
-  val Pair(
+
+  val (
    k,
    l) = (0, 0)
 }
