@@ -5,7 +5,7 @@ object Test extends DirectTest {
 
   override def extraSettings: String = "-usejavacp -Xprint:specialize -Ydelambdafy:method -d " + testOutput.path
 
-  override def code = "class Foo { val f = (param: Int) => param } "
+  override def code = "class Foo { val f = (param: String) => param } "
 
   override def show(): Unit = {
     Console.withErr(System.out) {
