@@ -6,9 +6,9 @@ object Test extends DirectTest {
   override def extraSettings: String = "-usejavacp -Xprint:uncurry -Ydelambdafy:method -Ystop-after:uncurry -d " + testOutput.path
 
   override def code = """class Foo {
-                        |  def bar(x: => Int) = x
+                        |  def bar(x: => String) = x
                         |  
-                        |  def foo = bar(1)
+                        |  def foo = bar("")
                         |}
                         |""".stripMargin.trim
 
