@@ -7,8 +7,8 @@ object Test extends JavapTest {
   def code = """
     |object Betty {
     | List(1,2,3) count (_ % 2 != 0)
-    | def f = List(1,2,3) filter (_ % 2 != 0) map (_ * 2)
-    | def g = List(1,2,3) filter (_ % 2 == 0) map (_ * 3) map (_ + 1)
+    | def f = List(1,2,3) filter ((x: Any) => true) map (x => "m1")
+    | def g = List(1,2,3) filter ((x: Any) => true) map (x => "m1") map (x => "m2")
     |}
     |:javap -fun Betty#g
   """.stripMargin
@@ -16,7 +16,7 @@ object Test extends JavapTest {
   // three anonfuns of Betty#g
   override def yah(res: Seq[String]) = {
     import PartialFunction.{ cond => when }
-    val r = """\s*private static final .* \$anonfun\$\d+\(.*""".r
+    val r = """.*final .* .*\$anonfun\$\d+\(.*""".r
     def filtered = res filter (when(_) { case r(_*) => true })
     3 == filtered.size
   }
