@@ -3,9 +3,7 @@ import scala.tools.partest.SessionTest
 
 object Test extends SessionTest {
   def session =
-s"""|Type in expressions to have them evaluated.
-    |Type :help for more information.
-    |
+s"""|
     |scala> :paste $pastie
     |Pasting file $pastie...
     |defined class Foo
