@@ -5,15 +5,14 @@ import scala.tools.partest.{ SessionTest, Welcoming }
 object Test extends SessionTest with Welcoming {
   def session =
 """Welcome to Scala
-Type in expressions to have them evaluated.
-Type :help for more information.
+Type in expressions for evaluation. Or try :help.
 
 scala> def f = throw new Exception("Uh-oh")
 f: Nothing
 
 scala> f
 java.lang.Exception: Uh-oh
-  at .f(<console>:7)
+  at .f(<console>:11)
   ... 69 elided
 
 scala> def f = throw new Exception("")
@@ -21,7 +20,7 @@ f: Nothing
 
 scala> f
 java.lang.Exception:
-  at .f(<console>:7)
+  at .f(<console>:11)
   ... 69 elided
 
 scala> def f = throw new Exception
@@ -29,7 +28,7 @@ f: Nothing
 
 scala> f
 java.lang.Exception
-  at .f(<console>:7)
+  at .f(<console>:11)
   ... 69 elided
 
 scala> :quit"""
@@ -37,7 +36,6 @@ scala> :quit"""
   // normalize the "elided" lines because the frame count depends on test context
   lazy val elided = """(\s+\.{3} )\d+( elided)""".r
   override def normalize(line: String) = line match {
-    case welcome(w)               => w
     case elided(ellipsis, suffix) => s"$ellipsis???$suffix"
     case s                        => s
   }
