@@ -164,6 +164,12 @@ class IteratorTest {
     assertEquals(1, y.next)
     assertFalse(x.hasNext)   // was true, after advancing underlying iterator
   }
+  // SI-9913
+  @Test def `span leading iterator finishes at state -1`(): Unit = {
+    val (yes, no) = Iterator(1, 2, 3).span(_ => true)
+    assertFalse(no.hasNext)
+    assertTrue(yes.hasNext)
+  }
   // SI-9623
   @Test def noExcessiveHasNextInJoinIterator: Unit = {
     var counter = 0
@@ -192,4 +198,22 @@ class IteratorTest {
     assertSameElements(exp, res)
     assertEquals(8, counter) // was 14
   }
+
+  // SI-9766
+  @Test def exhaustedConcatIteratorConcat: Unit = {
+    def consume[A](i: Iterator[A]) = {
+      while(i.hasNext) i.next()
+    }
+    val joiniter = Iterator.empty ++ Seq(1, 2, 3)
+    assertTrue(joiniter.hasNext)
+    consume(joiniter)
+    val concatiter = joiniter ++ Seq(4, 5, 6)
+    assertTrue(concatiter.hasNext)
+    consume(concatiter)
+    assertFalse(concatiter.hasNext)
+    val concatFromEmpty = concatiter ++ Seq(7, 8, 9)
+    assertTrue(concatFromEmpty.hasNext)
+    consume(concatFromEmpty)
+    assertFalse(concatFromEmpty.hasNext)
+  }
 }
