@@ -13,6 +13,12 @@ class PagedSeqTest {
     assertEquals(Seq('a'), PagedSeq.fromStrings(List.fill(5000)("a")).slice(4096, 4097))
   }
 
+  // should not NPE, and should be empty
+  @Test
+  def test_SI9480(): Unit = {
+    assertEquals(Seq(), PagedSeq.fromStrings(List("a")).slice(1))
+  }
+
   // Slices shouldn't read outside where they belong
   @Test
   def test_SI6519 {
